% Copyright (C) Emily Stephen 2014
function [] = computeNets_corr(varargin)
ip = inputParser;
ip.addRequired('bsecog',@isnumeric)
ip.addRequired('agg_ecog',@iscell)
ip.addRequired('params',@isstruct)
ip.parse(varargin{:});

bsecog = ip.Results.bsecog;
agg_ecog = ip.Results.agg_ecog;
params = ip.Results.params;

nettype_short = params.nettype_short;
nonsets = params.nonsets;
maxLags = params.maxLags;
Nbsreps = params.Nbsreps;
Nbsreps_outer = params.Nbsreps_outer;
swtOuterBS_Null = params.swtOuterBS_Null;
savename = params.savename;
blnFitNullBS = params.blnFitNullBS;

Nwin = size(bsecog,1);

%% Process correlations
% calculate xcorrs for bootstrap, keep trials separate for sampling
disp(['      Calculating xcorrs...'])
[cc_baseline,~,ac_baseline] = compute_xcorrs_new(bsecog,maxLags,false);
[cc_trial,~,ac_trial] = cellfun(@(x) compute_xcorrs_new(x,maxLags,false),agg_ecog,'UniformOutput',false);


%% Compute p-values
switch nettype_short,
    case {'maxAbsCorr'},
        fncStatistic = [];
    case {'zeroLagCorr'},
        fncStatistic = @(cc) cc(:,maxLags+1,:);
    case {'absZeroLagCorr'},
        fncStatistic = @(cc) abs(cc(:,maxLags+1,:));
end

% Create bootstrap samples from Baseline data
disp(['      Calculating bootstrap samples...'])
cc_bootstrap = corr_bootstrap_sample(cc_baseline,ac_baseline,Nbsreps,nonsets);
xhat_bootstrap = corr_applyFncStatistic(cc_bootstrap,fncStatistic,false,maxLags);

disp(['      Computing p-values...'])
[agg_rho,xhat,lag0,xhat_bs_mean,xhat_bs_std] = OneSampleNet(cc_trial,ac_trial,xhat_bootstrap,1:nonsets,fncStatistic,maxLags,blnFitNullBS);

if ~isempty(savename),
    save(savename,'xhat','-append','-v7.3');

    % Lags at which max occurs
    if strcmp(nettype_short,'maxAbsCorr'),
        save(savename,'lag0','-append','-v7.3')
    end

    if blnFitNullBS,
        save(savename,'xhat_bs_mean','xhat_bs_std','-append','-v7.3')
    else
        save(savename,'xhat_bootstrap','-append','-v7.3')
    end
end

%% Compute Network
disp(['      Calculating ' nettype_short ' networks...'])
tic
agg = network_loop_rho(agg_rho);
toc

if ~isempty(savename),
    % Observed Network
    save(savename,'agg','-append','-v7.3');
end
clear agg

%% Outer Bootstrap
switch swtOuterBS_Null,
    case 'same',
        xhat_bootstrap_outer = xhat_bootstrap;
    case 'resampleWInds',
        xhat_bootstrap_outer = [];
    case 'resample',
        disp(['      Calculating bootstrap samples for outer bootstrap networks...'])
        cc_bootstrap_outer = corr_bootstrap_sample(cc_baseline,ac_baseline,Nbsreps,nonsets,'blnDoubleSample',true);
        xhat_bootstrap_outer = corr_applyFncStatistic(cc_bootstrap_outer,fncStatistic,false,maxLags);
end
blnResampleNull = strcmp(swtOuterBS_Null,'resampleWInds');

agg_ebs = cell(1,Nbsreps_outer);
ebs_samples = zeros(nonsets,Nbsreps_outer);
parfor ibs_out=1:Nbsreps_outer,
    tic
    disp(['        Outer bootstrap sample #' num2str(ibs_out) '...'])
    % create bootstrap sample of trials
    % Sample from the set of trials with replacement
    ebs_sample = randi(nonsets,[nonsets,1]);

    % calculate xhat_ebs
    if blnResampleNull,
        % Create bootstrap null distribution
        cc_bootstrap_ebs = corr_bootstrap_sample(cc_baseline,ac_baseline,Nbsreps,nonsets,'inds',ebs_sample);
        xhat_bootstrap_ebs = corr_applyFncStatistic(cc_bootstrap_ebs,fncStatistic,false,maxLags);
    else
        xhat_bootstrap_ebs = xhat_bootstrap_outer;
    end

    [agg_rho_ebs] = OneSampleNet(cc_trial,ac_trial,xhat_bootstrap_ebs,ebs_sample,fncStatistic,maxLags,blnFitNullBS);

    agg_ebs{ibs_out} = network_loop_rho(agg_rho_ebs);

    ebs_samples(:,ibs_out) = ebs_sample;
    toc
end

if ~isempty(savename),
    % Outer Bootstrap
    if Nbsreps_outer>0,
        save(savename,'agg_ebs','ebs_samples','-append','-v7.3')
    end
    
    % Outer Null
    if strcmp(swtOuterBS_Null,'resample')
        save(savename,'xhat_bootstrap_outer','-append','-v7.3')
    end
end
clear agg_ebs ebs_samples xhat_bootstrap_outer cc_bootstrap


end

function [agg_rho,xhat,lag0,xhat_bs_mean,xhat_bs_std] = OneSampleNet(cc_trial,ac_trial,xhat_bootstrap,trialInds,fncStatistic,maxLags,blnFitNullBS)
    cc = cellfun(@(x,y) average_xcorr(x,y,trialInds),cc_trial,ac_trial,'UniformOutput',false);
    [xhat,lag0] = corr_applyFncStatistic(cc,fncStatistic,true,maxLags);
    
    if blnFitNullBS,
        % estimate mean and std error of bootstraps
        % xhat_bs_mean: (N x N)
        % xhat_bs_std: (N x N)
        xhat_bs_F = atanh(xhat_bootstrap);
        xhat_bs_mean = squeeze(mean(xhat_bs_F,1));
        xhat_bs_std = squeeze(std(xhat_bs_F,1,1));

        agg_rho = cellfun(@(xhat) compute_rho_OneSample(xhat,xhat_bs_mean,xhat_bs_std),xhat,'UniformOutput',false);
    else
        agg_rho = cellfun(@(xhat) compute_rho_OneSample(xhat,xhat_bootstrap),xhat,'UniformOutput',false);
        xhat_bs_mean = [];
        xhat_bs_std = [];
    end
end


