% Copyright (C) Emily Stephen 2014
%%%%%%%%
%NOTES.
%
% Based on compute_correlation_network_ES September 26, 2012
%
%%%%%%%%
%Compute a network using a grid of the test statistic and a collection of
%bootstrap grids for the test statistic.
%
% [C,xhat] = compute_network_ES(xhat,bsX,q)
%
%INPUTS:
%  xhat  = The grid of the test statistic for each node pair (N x N).
%           Should be square and symmetric
%  bsX   = bootstrap samples (Nbs x N x N) of the test statistic
%  q     = The q-value that determines the threshold of p-values to keep
%          (usually q=0.05).
%
%OUTPUT:
%  C   = Association matrix.  1=an edge, 0=no edge.  It's symmetric (i.e., C = C').

function [C] = compute_FDR(varargin)

% Parameter Parsing
ip = inputParser;
ip.addRequired('rho',@isnumeric);
ip.addRequired('q',@(x) x>0 && x<=1);
ip.addParamValue('blnTwoSided',false,@islogical);
ip.parse(varargin{:});

rho = ip.Results.rho;
q = ip.Results.q;
blnTwoSided = ip.Results.blnTwoSided;

N = size(rho,1);                %The number of nodes.
if any(size(rho)~=[N,N]),
    error('COMPUTE_NETWORK_ES:Xhaterror',...
      'Size of xhat input must be N x N');
end

%analyze the p-values in the 'rho' matrix.  Keep only the smallest
%(i.e., most significant) p-values.  The following bit of code thresholds
%the p-values using the FDR method.
%Input q is usually q = 0.05.

if blnTwoSided,
    fprintf(['Using two-sided test \n'])
    q = q/2;
end

% Test statistic in high tail
pvals = rho(isfinite(rho));
threshold = fdr(pvals,q);

%Make a network with edges corresponding to significant p-values.
C=zeros(N);
C(rho <= threshold)=1;

% test statistic on low tail
if blnTwoSided,
    threshold2 = fdr(1-pvals,q);
    C(1-rho <= threshold2) = -1;
end

C = C + C';

end

function threshold = fdr(pvals,q)
    pvals = sort(pvals);
    np = length(pvals);
    
    BHcomp = pvals' <= (1:np)*q/np;
    k = find(BHcomp,1,'last');
    if isempty(k)
        threshold = 0.0;
    else
        threshold = pvals(k);
    end
end