% Copyright (C) Emily Stephen 2014
%%%%%%%%
%NOTES.
%
% Based on compute_correlation_network_ES September 26, 2012
%
%%%%%%%%
%Compute a network using a grid of the test statistic and a collection of
%bootstrap grids for the test statistic.
%
% [rho] = compute_network_ES(xhat,xhat_bootstrap)
% [rho] = compute_network_ES(xhat,xhat_bs_mean,xhat_bs_std)
%
%INPUTS:
%  xhat  = The grid of the test statistic for each node pair (N x N).
%           Should be square and symmetric
%  bsX   = bootstrap samples (Nbs x N x N) of the test statistic
%
%OUTPUT:
%  rho   = P-value matrix

function [rho] = compute_rho_OneSample(varargin)

% Parameter Parsing
ip = inputParser;
ip.addRequired('xhat',@isnumeric);
ip.addRequired('bs_arg1',@isnumeric); % either xhat_bootstrap or xhat_bs_mean
ip.addOptional('bs_arg2',[],@isnumeric); % if xhat_bs_mean above, then xhat_bs_std
ip.parse(varargin{:});

xhat = ip.Results.xhat;
blnFitNullBS = ~isempty(ip.Results.bs_arg2);
if blnFitNullBS,
    bsX_mean = ip.Results.bs_arg1;
    bsX_std = ip.Results.bs_arg2;
else
    bsX = ip.Results.bs_arg1;
end

N = size(xhat,1);                %The number of nodes.
if any(size(xhat)~=[N,N]),
    error('COMPUTE_NETWORK_ES:Xhaterror',...
      'Size of xhat input must be N x N');
end

if blnFitNullBS,
    if any(size(bsX_mean)~=[N,N])||any(size(bsX_std)~=[N,N]),
        error('COMPUTE_NETWORK_ES:bsXerror',...
          'Size of bsX_mean and bsX_std must be N x N');
    end
else
    nbs = size(bsX,1);             % The number of bootstrap samples
    if any(size(bsX)~=[nbs,N,N]),
        error('COMPUTE_NETWORK_ES:bsXerror',...
          'Size of bsX input must be Nbs x N x N');
    end
end

% bootstrap comparison
rho = zeros(N,N);
if blnFitNullBS,
    xhat_F = atanh(xhat);
    if any(~isreal(xhat_F)),
        xhat_F = abs(xhat_F);
        bsX_mean = abs(bsX_mean);
        bsX_std = abs(bsX_std);
    end
    rho = 1-normcdf(xhat_F,bsX_mean,bsX_std);
else
    for e1=1:N,
        for e2=(e1+1):N,
            bsvals = bsX(:,e1,e2);
            phigher = sum(bsvals>xhat(e1,e2))/nbs;
            rho(e1,e2) = phigher;
        end
    end
    rho(rho==0) = 1/nbs;
end
rho(find(tril(ones(length(rho)))==1))=NaN;

