% Copyright (C) Emily Stephen 2014
function SNDFout=basic_preprocess(varargin)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Preprocessing ECoG data:
%
% Keeps good channels as defined by the good_channels variable in the vars
%   field. 
% High-pass filter (3rd order Butterworth Filter, zero-phase)
% Low-pass filter (3rd order Butterworth Filter, zero-phase)
% Notch filter (3rd order butter, zero-phase) 
% Then apply CAR.
%
% INPUT:
% SNDFin - the SNDF CNT struct to be processed
% lp_freq - the frequency to low-pass filter ([] for none)
% hp_freq - the frequency to high-pass filter ([] for none)
% notch_freqs - array of notch filter frequencies ([] for none)
%
% Parameters:
% 'SelectGoodChannels' - boolean, whether to select channels using the
%   good_channels variable in the vars field. default: true
% 'post_fs' - the sampling rate to downsample to. 
%   default (lp_freq specified): pick_ds(pre_fs,lp_freq)
%   default (no lp_freq): pre_fs
%
% OUTPUT:
% Optional cell array of output variables:
% SNDFout - Preprocessed SNDF CNT struct
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Set input parameters
ip=inputParser;
ip.addRequired('SNDFin',@isstruct);
ip.addRequired('lp_freq',@(x) isnumeric(x)||isempty(x))
ip.addRequired('hp_freq',@(x) isnumeric(x)||isempty(x))
ip.addRequired('notch_freqs',@isnumeric)
ip.addParamValue('SelectGoodChannels',true,@islogical);
ip.addParamValue('post_fs',[],@isnumeric)
ip.addParamValue('blnCAR',true,@islogical);
ip.addParamValue('CAR_sets',{},@iscell);
ip.parse(varargin{:})

SNDFin = ip.Results.SNDFin;
lp_freq = ip.Results.lp_freq; %Hz
hp_freq = ip.Results.hp_freq; %Hz
notch_freqs = ip.Results.notch_freqs; %Hz
blnCAR = ip.Results.blnCAR;
CAR_sets = ip.Results.CAR_sets;

% Create useful variables
if ip.Results.SelectGoodChannels
    good_channels = SNDFin.vars.good_channels;
else
    good_channels = 1:size(SNDFin.SampValues,2);
end

if blnCAR && isempty(CAR_sets),
    CAR_sets = {good_channels};
end

% check for problems with CAR_sets, e.g. electrodes that are not in
% good_channels, not including all electrodes in good_channels, or
% electrodes that are included in more than one set.
if blnCAR,
    if length(intersect([CAR_sets{:}],good_channels))~=length(good_channels) ||...
            sum(cellfun(@length,CAR_sets))~=length(good_channels),
        error('BASIC_PREPROCESS:CARerror',...
              'CAR_sets mismatch with good_channels');
    end
end

pre_Fs = SNDFin.SampFreq;

% Find downsample amount and post Fs
if isempty(ip.Results.post_fs),
    if isempty(lp_freq),
        ds_amt = 1;
        post_Fs = pre_Fs;
    else
%         ds_amt = 2^(nextpow2(pre_Fs/(lp_freq*2)) - 1);
        [ds_amt,post_Fs] = pick_ds(pre_Fs,lp_freq);
    end
else
    post_Fs = ip.Results.post_fs;
    ds_amt = pre_Fs/post_Fs;
end

% Check that ds_amt and post_Fs are integers
if rem(post_Fs,1)~=0 || rem(ds_amt,1)~=0,
    error('post_Fs and ds_amt must both be integers')
end

post_N = length(good_channels);
post_nyq = post_Fs/2;

% Process each fragment separately
F = length(SNDFin.FragLengths);
post_T = zeros(F,1);
post_signal = [];
for fi = 1:F,
    %% Find fragment
    pre_T = SNDFin.FragLengths(fi);
    pre_signal = cnt_getfrag(SNDFin,fi);
    
    post_T(fi) = ceil(pre_T/ds_amt);

    %% Keep only good channels
    good_signal = pre_signal(:,good_channels);
    clear pre_signal

    %% Downsample
    % Decimate signal
    ds_signal = zeros(post_T(fi),post_N);
    for i=1:post_N,
        ds_signal(:,i) = decimate(good_signal(:,i),ds_amt);
    end
    clear good_signal

    %% Highpass filter
    if isempty(hp_freq)
        hp_signal = ds_signal;
    else
        [b,a] = butter(3,hp_freq/post_nyq,'high');
        % freqz(b,a,[],post_Fs)

        hp_signal = zeros(post_T(fi),post_N);
        for i=1:post_N,
            hp_signal(:,i) = filtfilt(b,a,ds_signal(:,i));
        end
    end
    clear ds_signal

    %% Lowpass filter
    if isempty(lp_freq),
        lp_signal = hp_signal;
    else
        [b,a] = butter(3,lp_freq/post_nyq,'low');
        % freqz(b,a,[],post_Fs)

        lp_signal = zeros(post_T(fi),post_N);
        for i=1:post_N,
            lp_signal(:,i) = filtfilt(b,a,hp_signal(:,i));
        end
    end
    clear hp_signal
    
    %% Notch Filters
    notch_signal = lp_signal;
    for ni=1:length(notch_freqs),
        [b,a] = butter(3,[notch_freqs(ni)-.5 notch_freqs(ni)+.5]/post_nyq,'stop');
        % freqz(b,a,[],post_Fs)

        for i=1:post_N,
            notch_signal(:,i) = filtfilt(b,a,notch_signal(:,i));
        end
    end
    clear lp_signal

    %% Apply CAR
    if blnCAR,
        CAR_signal = nan(size(notch_signal));
        for ci=1:length(CAR_sets),
            disp(['CAR_set #' num2str(ci) ': ' num2str(CAR_sets{ci})])
            [~,carinds] = intersect(good_channels,CAR_sets{ci}); % channel indices of the CAR set
            if length(carinds)~=length(CAR_sets{ci}),
                error('BASIC_PREPROCESS:CARerror',...
                      'CAR sets mismatch with good_channels');
            end
            mean_ref = mean(notch_signal(:,carinds),2);
            CAR_signal(:,carinds) = notch_signal(:,carinds) - repmat(mean_ref,1,length(carinds));

            % redefine CAR_sets for logging (in case the intersect removed
            % channels)
            CAR_sets{ci} = good_channels(carinds);
        end
        if max(isnan(CAR_signal))~=0,
            error('BASIC_PREPROCESS:CARerror',...
                  'CAR signal not completely specified');
        end
    else
        CAR_signal = notch_signal;
    end
    clear notch_signal
    
    %% Append to output signal
    post_signal = [post_signal; CAR_signal];
    clear CAR_signal
end

clear SNDFin.SampValues

%% Plot and save figures
% fig1=figure;
% plot(data.outvar1);
% print(fig1,'-dpng','-r300','fig1.png');
% saveas(fig1,'fig1.fig');

%% Generate output SNDF struct
SNDFout = struct('SampValues',post_signal,...
                 'SampFreq',post_Fs,...
                 'SubjectID',SNDFin.SubjectID,...
                 'SampTimes',SNDFin.SampTimes,...
                 'FragLengths',post_T,...
                 'DataUnits',SNDFin.DataUnits,...
                 'vars',SNDFin.vars,...
                 'TimeUnits','s',...
                 'Log',{SNDFin.Log});
SNDFout.ChLbl = {SNDFin.ChLbl{good_channels}};


% Add a Log entry
pos=size(SNDFout.Log,1)+1;
SNDFout.Log{pos,1}=mfilename;
SNDFout.Log{pos,2}=datestr(clock);
SNDFout.Log{pos,3}=['HP Freq: ' num2str(hp_freq)];
SNDFout.Log{pos,4}=['LP Freq: ' num2str(lp_freq)];
SNDFout.Log{pos,5}=['Notch Freqs: ' num2str(notch_freqs)];
SNDFout.Log{pos,6}=['ds_amt: ' num2str(ds_amt)];
SNDFout.Log{pos,7}=['blnCAR: ' num2str(blnCAR)];
for i=1:length(CAR_sets),
    SNDFout.Log{pos,7+i}=['CAR_sets{' num2str(i) '}: [' num2str(CAR_sets{i}) ']'];
end

end