% Copyright (C) Emily Stephen 2014
function varargout=utterances_fromCSV_wSegValues(varargin)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Create utterance DSC struct from CSV file (containing utterance onset
% times) and CNT struct (containing ECoG data)
%
%
% INPUT:
% csv_onsetfile - the path to the csv file containing the utterance onset
%           times ([] for open dialog)
% cntfile - the path to the SNDF cnt struct to be cut down into utterance
%            segments ([] for open dialog)
% outputfile - where to save the results ([] for no save)
% before_s - minimum amount of silence before voice onset (sec)
% after_s - minimum amount of voicing after voice onset (sec)
%
% OUTPUT:
% SNDFout - Preprocessed SNDF DSC struct
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Set input parameters
ip=inputParser;
ip.addRequired('csv_onsetfile',@(x)(isempty(x) || ischar(x) || isnumeric(x)));
ip.addRequired('cntfile',@(x)(isempty(x) || ischar(x)));
ip.addRequired('outputfile',@(x)(isempty(x) || ischar(x)));
ip.addRequired('before_s',@isnumeric);
ip.addRequired('after_s',@isnumeric);
ip.parse(varargin{:})
csv_onsetfile = ip.Results.csv_onsetfile;
cntfile = ip.Results.cntfile;
outputfile = ip.Results.outputfile;
before_s = ip.Results.before_s;
after_s = ip.Results.after_s;

if (nargout == 0) && isempty(outputfile)
    error('UTTERANCES_FROMCSV_WSEGVALUES:NoOutput',...
          'utterances_fromCSV_wSegValues run with no saving and no outputs');
end

% Get utterance file if none specified
if isempty(csv_onsetfile)
    [f,p] = uigetfile({'*.csv*','CSV Files (*.csv*)'}, ...
                      'Select Utterance Onset File',...
                      'multiselect','off');
    if p==0
        if nargout > 0
            varargout = {[]};
        end
        return
    else
        csv_onsetfile = fullfile(p,f);
    end
end

% Get cnt file if none specified
if isempty(cntfile)
    [f,p] = uigetfile({'*.mat*','SNDF CNT Files (*.mat*)'}, ...
                      'Select SNDF CNT File',...
                      'multiselect','off');
    if p==0
        if nargout > 0
            varargout = {[]};
        end
        return
    else
        cntfile = fullfile(p,f);
    end
end


% don't save if outputfolder is empty
blnsave = ~isempty(outputfile);

if ischar(csv_onsetfile),
    onset_times = sort(load(csv_onsetfile));
else
    onset_times = csv_onsetfile;
end
nonsets = length(onset_times);

cnt = load(cntfile);

%% Find Trials

N = length(cnt.vars.good_channels);

% find trials
window = round(-before_s*cnt.SampFreq):round(after_s*cnt.SampFreq);
ecog = zeros(nonsets,length(window),N);
ta = cnt_timeaxis(cnt);
for u=1:nonsets,
    % Find ecog during utterance
    ontime = onset_times(u);
    
    inds = find(ta>ontime,1,'first')+window;
    
    if ~cnt_checktime(cnt,ta(inds)),
        error('UTTERANCES_FROMCSV_WSEGVALUES:MissingData',...
          ['Utterance #' num2str(u) ' requires data that is not contained within the cnt file']);
    end
    
    ecog(u,:,:) = cnt.SampValues(inds,:);
end

%% Save results

% use only onsets
EvtLbl = {'Onset'};
SNDFout = struct('EvtTimes',onset_times,...
                 'EvtID',ones(size(onset_times)),...
                 'EvtLbl',{EvtLbl},...
                 'vars',cnt.vars,...
                 'TimeUnits','s',...
                 'Log',{cnt.Log});


% Save extra variables
SNDFout.SegValues = ecog;
SNDFout.ChLbl = cnt.ChLbl;
SNDFout.SegSampFreq = cnt.SampFreq;
SNDFout.SegMask = window;
SNDFout.LinkedCntData = {cntfile};
SNDFout.DataUnits = cnt.DataUnits;

% Add Log entry
pos=size(SNDFout.Log,1)+1;
SNDFout.Log{pos,1}=mfilename;
SNDFout.Log{pos,2}=datestr(clock);
SNDFout.Log{pos,3}=['csv file: ' csv_onsetfile];
SNDFout.Log{pos,4}=['before_s: ' num2str(before_s) 's'];
SNDFout.Log{pos,5}=['after_s: ' num2str(after_s) 's'];

if blnsave
    fprintf(1,'Saving: %s\n',outputfile);
    save(outputfile,'-struct','SNDFout','-v7.3');
end

if nargout == 1
    varargout = {SNDFout};
end


